package ovh.herisson.Clyde.EndPoints;


import jakarta.servlet.http.HttpServletRequest;
import org.springframework.http.HttpStatus;

import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.UserService;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.User;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;


@RestController
@CrossOrigin(origins = "http://localhost:5173")
public class UserController {

    private final UserService userService;
    private final AuthenticatorService authServ;
    public UserController(UserService userService, AuthenticatorService authServ){
        this.userService = userService;
        this.authServ = authServ;
    }

    @GetMapping("/user")
    public ResponseEntity<Object[]> getUser(@RequestHeader("Authorization") String authorization){

        if (authorization == null) return new UnauthorizedResponse<>(null);
        User user = authServ.getUserFromToken(authorization);
        if (user == null) return new UnauthorizedResponse<>(null);

        return new ResponseEntity<>(userWithoutPassword(user), HttpStatus.OK);
    }

    @PostMapping("/user")
    public ResponseEntity<String> postUser(@RequestBody User user,@RequestHeader("Authorization") String authorization){

        if (!isSecretaryOrAdmin(authorization))
            return new UnauthorizedResponse<>(null);

        userService.save(user);
        return new ResponseEntity<>(String.format("Account created with ID:%s",user.getRegNo()),HttpStatus.CREATED);
    }

    @GetMapping("/users")
    public ResponseEntity<Iterable<Object[]>> getAllUsers(@RequestHeader("Authorization") String authorization){

        if (!isSecretaryOrAdmin(authorization))
            return new UnauthorizedResponse<>(null);

        Iterable<User> users = userService.getAll();
        ArrayList<Object[]> withoutPassword = new ArrayList<>();

        for (User u :users){
            withoutPassword.add(userWithoutPassword(u));
        }
        return new ResponseEntity<>(withoutPassword, HttpStatus.OK);
    }
    @PatchMapping("/user")
    public ResponseEntity<String> patchUser(@RequestBody Map<String,Object> updates, @RequestHeader("Authorization") String authorization) {

        if (authorization == null) return new UnauthorizedResponse<>(null);

        User poster = authServ.getUserFromToken(authorization);
        if (poster == null) {return new UnauthorizedResponse<>("bad authorization");}

        if (!userService.modifyData(poster, updates, poster))
            return new UnauthorizedResponse<>("there was an issue with the updates requested");

        return new ResponseEntity<>("data modified", HttpStatus.OK);
    }


        /** return user's data except password
         * @param user the user to return
         * @return all the user data without the password
         */
    private Object[] userWithoutPassword(User user){
        return new Object[] {user.getRegNo(),user.getFirstName(),user.getLastName(),user.getBirthDate(),user.getCountry(),user.getAddress(),user.getRole()};
    }

    private boolean isSecretaryOrAdmin(String authorization){
        if (authorization ==null)
            return false;

        User poster = authServ.getUserFromToken(authorization);
        if (poster == null) return false;

        return poster.getRole() == Role.Secretary && poster.getRole() == Role.Admin;
    }
}

